@@ -2,7 +2,7 @@ import click
 
 from deployfish.cli.adapters.utils import handle_model_exceptions, print_render_exception
 from deployfish.config import get_config
-from deployfish.core.models import Cluster, AutoscalingGroup, CloudWatchLogGroup, Service, StandaloneTask
+from deployfish.core.models import Cluster, AutoscalingGroup, CloudWatchLogGroup, StandaloneTask
 from deployfish.core.waiters.hooks import ECSDeploymentStatusWaiterHook, ECSTaskStatusHook
 from deployfish.exceptions import RenderException, ConfigProcessingFailed
 from deployfish.typing import FunctionTypeCommentParser
@@ -52,6 +52,103 @@ class HelperTaskCommandMixin(object):
 # Command mixins
 # ====================
 
+# Cluster/Service
+# ---------------
+
+class ClickListRunningTasksCommandMixin(object):
+
+    list_running_tasks_ordering = None
+    list_running_tasks_result_columns = {}
+    list_running_tasks_renderer_classes = {
+        'table': TableRenderer,
+        'detail': TemplateRenderer,
+        'json': JSONRenderer
+    }
+
+    @classmethod
+    def list_running_tasks_display_option_kwargs(cls):
+        """
+        Return the appropriate kwargs for `click.option('--display', **kwargs)` for the renderer options we've defined
+        for the list endpoint.
+
+        :rtype: dict
+        """
+        render_types = list(cls.list_running_tasks_renderer_classes.keys())
+        default = render_types[0]
+        kwargs = {
+            'type': click.Choice(render_types),
+            'default': default,
+            'help': "Render method for listing {} objects. Choices: {}.  Default: {}.".format(
+                cls.model.__name__,
+                ', '.join(render_types),
+                default
+            )
+        }
+        return kwargs
+
+    @classmethod
+    def add_list_running_tasks_click_command(cls, command_group):
+        """
+        Build a fully specified click command for listing running tasks for an ECS service or cluster, and add it to the
+        click command group `command_group`.  Return the properly wrapped function object.
+
+        :param command_group function: the click command group function to use to register our click command
+
+        :rtype: function
+        """
+        def list_running_tasks(ctx, *args, **kwargs):
+            if cls.model.config_section is not None:
+                try:
+                    ctx.obj['config'] = get_config(**ctx.obj)
+                except ConfigProcessingFailed as e:
+                    ctx.obj['config'] = e
+            ctx.obj['adapter'] = cls()
+            click.secho(ctx.obj['adapter'].list_running_tasks(kwargs['identifier'], kwargs['display']))
+
+        pk_description = cls.get_pk_description()
+        list_running_tasks.__doc__ = """
+List the running tasks associated with a {object_name} in AWS.
+
+{pk_description}
+""".format(
+            pk_description=pk_description,
+            object_name=cls.model.__name__
+        )
+
+        function = print_render_exception(list_running_tasks)
+        function = click.pass_context(function)
+        function = click.option('--display', **cls.list_running_tasks_display_option_kwargs())(function)
+        function = click.argument('identifier')(function)
+        function = command_group.command(
+            'running-tasks',
+            short_help='List the running tasks for a {object_name} in AWS.'.format(
+                object_name=cls.model.__name__
+            )
+        )(function)
+        return function
+
+    @handle_model_exceptions
+    def list_running_tasks(self, identifier, display):
+        assert display in self.list_running_tasks_renderer_classes, \
+            'list running tasks: "{}" is not a valid rendering option'.format(
+                display
+            )
+        obj = self.get_object_from_aws(identifier)
+        results = obj.running_tasks
+        if not results:
+            return('No results.')
+        else:
+            if display == 'table':
+                results = self.list_running_tasks_renderer_classes[display](
+                    self.list_running_tasks_result_columns,
+                    ordering=self.list_running_tasks_ordering
+                ).render(results)
+            else:
+                results = self.list_running_tasks_renderer_classes[display]().render(results)
+            results = '\n' + results + '\n'
+            return results
+
+
 # Service
 # ---------
 
@@ -82,19 +179,25 @@ class ClickScaleInstancesCommandMixin(object):
 
         pk_description = cls.get_pk_description()
         scale_instances.__doc__ = """
-Change the number of instances for a {object_name} in AWS.
+Change the number of instances for an ECS cluster in AWS.
+
+We'll try to detect the name of the autoscaling group by one of two methods:
+
+  * Look for a tag on the cluster named "deployfish:autoscalingGroup"
+
+  * Look for the tag named "aws:autoscalingGroup" on one of the running container instances
 
 {pk_description}
 
 COUNT is an integer.
-""".format(pk_description=pk_description, object_name=cls.model.__name__)
+""".format(pk_description=pk_description)
 
         function = print_render_exception(scale_instances)
         function = click.pass_context(function)
         function = click.option(
             '--force/--no-force',
             default=False,
-            help='Force the {object_name} to scale outside its MinCount or MaxCount'.format(
+            help='Force the Cluster to scale outside its MinCount or MaxCount'.format(
                 object_name=cls.model.__name__
             )
         )(function)
@@ -102,7 +205,7 @@ COUNT is an integer.
         function = click.argument('identifier')(function)
         function = command_group.command(
             'scale',
-            short_help='Change the number of instances for a {object_name} in AWS.'.format(
+            short_help='Change the number of instances for a Cluster in AWS.'.format(
                 object_name=cls.model.__name__
             )
         )(function)
@@ -288,6 +391,70 @@ Service is completely wedged.
         return click.style('\n\nRestarted tasks for {}("{}").'.format(self.model.__name__, obj.pk), fg='green')
 
 
+class ClickListServiceRelatedTasksCommandMixin(ClickUpdateObjectCommandMixin):
+    @classmethod
+    def add_list_related_tasks_click_command(cls, command_group):
+        """
+        Build a fully specified click command for listing StandaloneTasks related to a Service from what we have in our
+        deployfish.yml file, and add it to the click command group `command_group`.  Return the function object.
+
+        :param command_group function: the click command group function to use to register our click command
+
+        :rtype: function
+        """
+        if cls.model.config_section is None:
+            raise cls.ReadOnly(
+                '{} objects are read only. If you want them to be read/write, define the '
+                '"config_section" class attribute on the model to be the section in deployfish.yml '
+                'where configuration info can be found for them.'
+            )
+
+        def list_related_tasks(ctx, *args, **kwargs):
+            try:
+                ctx.obj['config'] = get_config(**ctx.obj)
+            except ConfigProcessingFailed as e:
+                raise RenderException(str(e))
+            ctx.obj['adapter'] = cls()
+            ctx.obj['adapter'].list_related_standalone_tasks(kwargs.pop('identifier'))
+        list_related_tasks.__doc__ = """
+List StandaloneTasks related to a Service from what we have in our deployfish.yml file.
+
+NOTE: This lists tasks defined under the top level 'tasks:' section in deployfish.yml.  ServiceHelperTasks -- those
+defined by a 'tasks:' section under the Service definition will not be listed here.
+
+IDENTIFIER is a string that looks like one of:
+
+    * Service.name
+
+    * Service.environment
+
+"""
+        function = print_render_exception(list_related_tasks)
+        function = click.pass_context(function)
+        function = click.argument('identifier')(function)
+        function = command_group.command(
+            'list-related-tasks',
+            short_help='List StandaloneTasks related to a Service from configuration in deployfish.yml'
+        )(function)
+        return function
+
+    def list_related_standalone_tasks(self, service_identifier, **kwargs):
+        service = self.get_object_from_deployfish(
+            service_identifier,
+            factory_kwargs=self.factory_kwargs.get('list_related_tasks', {})
+        )
+        config = get_config()
+        tasks = []
+        for task_data in config.cooked['tasks']:
+            if 'service' in task_data:
+                if (task_data['service'] == service.pk or task_data['service'] == service.name):
+                    tasks.append(task_data['name'])
+        tasks.sort()
+        if tasks:
+            for task in tasks:
+                click.echo(task)
+
+
 class ClickUpdateServiceRelatedTasksCommandMixin(ClickUpdateObjectCommandMixin):
     @classmethod
     def add_update_related_tasks_click_command(cls, command_group):
@@ -847,6 +1014,121 @@ SERVICE_IDENTIFIER is a string that looks like one of:
         return click.style('\nDone.', fg='yellow')
 
 
+class ClickDisableHelperTaskMixin(object):
+
+    @classmethod
+    def add_disable_task_schedule_click_command(cls, command_group):
+        """
+        Build a fully specified click command for disabling the schedule rule for a task, and add it to the click
+        command group `command_group`.  Return the function object.
+
+        :param command_group function: the click command group function to use to register our click command
+
+        :rtype: function
+        """
+        def disable_schedule(ctx, *args, **kwargs):
+            if cls.model.config_section is not None:
+                try:
+                    ctx.obj['config'] = get_config(**ctx.obj)
+                except ConfigProcessingFailed:
+                    pass
+            ctx.obj['adapter'] = cls()
+            click.secho(ctx.obj['adapter'].disable_schedule(
+                kwargs['service_identifier'],
+                kwargs['helper_task_name'],
+            ))
+
+        pk_description = cls.get_pk_description(name='SERVICE_IDENTIFIER')
+        pk_description = cls.get_pk_description()
+        disable_schedule.__doc__ = """
+If a ServiceHelperTask in AWS has a schedule rule and that rule is currently enabled, disable it.
+
+{pk_description}
+
+""".format(pk_description=pk_description, object_name=cls.model.__name__)
+
+        function = print_render_exception(disable_schedule)
+        function = click.pass_context(function)
+        function = click.argument('helper_task_name')(function)
+        function = click.argument('service_identifier')(function)
+        function = command_group.command(
+            'disable',
+            short_help='Disable the schedule for a ServiceHelperTask.'
+        )(function)
+        return function
+
+    @handle_model_exceptions
+    def disable_schedule(self, service_pk, command_name, **kwargs):
+        command = self.get_task(service_pk, command_name)
+        if command.schedule is None:
+            return click.style(
+                f'ABORT: ServiceHelperTask("{command_name}") has no schedule; disabling only affects schedules.',
+                fg='yellow'
+            )
+        command.disable_schedule()
+        if command.schedule.enabled:
+            return click.style(f'ServiceHelperTask("{command_name}") state is now ENABLED.', fg='red')
+        else:
+            return click.style(f'ServiceHelperTask("{command_name}") state is now DISABLED.', fg='green')
+
+
+class ClickEnableHelperTaskMixin(object):
+
+    @classmethod
+    def add_enable_task_schedule_click_command(cls, command_group):
+        """
+        Build a fully specified click command for enabling the schedule rule for a task, and add it to the click
+        command group `command_group`.  Return the function object.
+
+        :param command_group function: the click command group function to use to register our click command
+
+        :rtype: function
+        """
+        def enable_schedule(ctx, *args, **kwargs):
+            if cls.model.config_section is not None:
+                try:
+                    ctx.obj['config'] = get_config(**ctx.obj)
+                except ConfigProcessingFailed:
+                    pass
+            ctx.obj['adapter'] = cls()
+            click.secho(ctx.obj['adapter'].enable_schedule(
+                kwargs['service_identifier'],
+                kwargs['helper_task_name'],
+            ))
+
+        pk_description = cls.get_pk_description(name='SERVICE_IDENTIFIER')
+        enable_schedule.__doc__ = """
+If a ServiceHelperTask in AWS has a schedule rule and that rule is currently disabled, enable it.
+
+{pk_description}
+
+""".format(pk_description=pk_description, object_name=cls.model.__name__)
+
+        function = print_render_exception(enable_schedule)
+        function = click.pass_context(function)
+        function = click.argument('helper_task_name')(function)
+        function = click.argument('service_identifier')(function)
+        function = command_group.command(
+            'enable',
+            short_help='Enable the schedule for a ServiceHelperTask.'
+        )(function)
+        return function
+
+    @handle_model_exceptions
+    def enable_schedule(self, service_pk, command_name, **kwargs):
+        command = self.get_task(service_pk, command_name)
+        if command.schedule is None:
+            return click.style(
+                f'ABORT: ServiceHelperTask("{command_name}") has no schedule; enabling only affects schedules.',
+                fg='yellow'
+            )
+        command.enable_schedule()
+        if command.schedule.enabled:
+            return click.style(f'ServiceHelperTask("{command_name}") state is now ENABLED.', fg='green')
+        else:
+            return click.style(f'ServiceHelperTask("{command_name}") state is now DISABLED.', fg='red')
+
+
 # StandaloneTasks
 # ---------------
 
@@ -1033,7 +1315,7 @@ If a StandaloneTask uses "awslogs" as its logDriver, list the available log stre
         function = click.argument('identifier')(function)
         function = command_group.command(
             'list',
-            short_help='List awslogs log streams for a ServiceStandaloneTask.'
+            short_help='List awslogs log streams for a StandaloneTask.'
         )(function)
         return function
 
@@ -1054,3 +1336,115 @@ If a StandaloneTask uses "awslogs" as its logDriver, list the available log stre
             'Last Event': {'key': 'lastEventTimestamp', 'datatype': 'timestamp', 'default': ''},
         }
         return '\n' + TableRenderer(columns, ordering='-Created').render(streams) + '\n'
+
+
+class ClickDisableStandaloneTaskMixin(object):
+
+    @classmethod
+    def add_disable_schedule_click_command(cls, command_group):
+        """
+        Build a fully specified click command for disabling the schedule rule for a task, and add it to the click
+        command group `command_group`.  Return the function object.
+
+        :param command_group function: the click command group function to use to register our click command
+
+        :rtype: function
+        """
+        def disable_schedule(ctx, *args, **kwargs):
+            if cls.model.config_section is not None:
+                try:
+                    ctx.obj['config'] = get_config(**ctx.obj)
+                except ConfigProcessingFailed:
+                    pass
+            ctx.obj['adapter'] = cls()
+            click.secho(ctx.obj['adapter'].disable_schedule(kwargs['identifier']))
+
+        pk_description = cls.get_pk_description()
+        disable_schedule.__doc__ = """
+If a StandaloneTask in AWS has a schedule rule and that rule is currently enabled, disable it.
+
+{pk_description}
+
+""".format(pk_description=pk_description, object_name=cls.model.__name__)
+
+        function = print_render_exception(disable_schedule)
+        function = click.pass_context(function)
+        function = click.argument('identifier')(function)
+        function = command_group.command(
+            'disable',
+            short_help='Disable the schedule for a StandaloneTask.'
+        )(function)
+        return function
+
+    @handle_model_exceptions
+    def disable_schedule(self, pk, **kwargs):
+        obj = self.get_object_from_aws(pk)
+        if obj.schedule is None:
+            return click.style(
+                f'ABORT: StandaloneTask("{pk}") has no schedule; disabling only affects schedules.',
+                fg='yellow'
+            )
+        obj.disable_schedule()
+        if obj.schedule.enabled:
+            return click.style(f'StandaloneTask("{pk}") state is now ENABLED.', fg='red')
+        else:
+            return click.style(f'StandaloneTask("{pk}") state is now DISABLED.', fg='green')
+
+
+class ClickEnableStandaloneTaskMixin(object):
+
+    @classmethod
+    def add_enable_schedule_click_command(cls, command_group):
+        """
+        Build a fully specified click command for enabling the schedule rule for a task, and add it to the click
+        command group `command_group`.  Return the function object.
+
+        :param command_group function: the click command group function to use to register our click command
+
+        :rtype: function
+        """
+        def enable_schedule(ctx, *args, **kwargs):
+            if cls.model.config_section is not None:
+                try:
+                    ctx.obj['config'] = get_config(**ctx.obj)
+                except ConfigProcessingFailed:
+                    pass
+            ctx.obj['adapter'] = cls()
+            lines = ctx.obj['adapter'].enable_schedule(
+                kwargs['identifier'],
+            )
+            if lines.count('\n') > 40:
+                click.echo_via_pager(lines)
+            else:
+                click.echo(lines)
+
+        pk_description = cls.get_pk_description()
+        enable_schedule.__doc__ = """
+If a StandaloneTask in AWS has a schedule rule and that rule is currently disabled, enable it.
+
+{pk_description}
+
+""".format(pk_description=pk_description, object_name=cls.model.__name__)
+
+        function = print_render_exception(enable_schedule)
+        function = click.pass_context(function)
+        function = click.argument('identifier')(function)
+        function = command_group.command(
+            'enable',
+            short_help='Enable the schedule for a StandaloneTask.'
+        )(function)
+        return function
+
+    @handle_model_exceptions
+    def enable_schedule(self, pk, **kwargs):
+        obj = self.get_object_from_aws(pk)
+        if obj.schedule is None:
+            return click.style(
+                f'ABORT: StandaloneTask("{pk}") has no schedule; enabling only affects schedules.',
+                fg='yellow'
+            )
+        obj.enable_schedule()
+        if obj.schedule.enabled:
+            return click.style(f'StandaloneTask("{pk}") state is now ENABLED.', fg='green')
+        else:
+            return click.style(f'StandaloneTask("{pk}") state is now DISABLED.', fg='red')
