@@ -1,3 +1,5 @@
+from itertools import cycle
+
 import click
 from tabulate import tabulate
 
@@ -25,7 +27,7 @@ class GetSSHTargetMixin(object):
                 for i, target in enumerate(obj.ssh_targets):
                     rows.append([
                         i + 1,
-                        click.style(target.tags['Name'], fg='cyan'),
+                        click.style(target.name, fg='cyan'),
                         target.pk,
                         target.ip_address
                     ])
@@ -44,36 +46,33 @@ class GetSSHTargetMixin(object):
 class GetExecTargetMixin(object):
 
     def get_exec_target(self, obj, choose=False):
+        """
+        This depends on ``obj`` having a method named ``running_tasks``.
+        """
         target = None
         container_name = None
         if choose:
-            if obj.ssh_targets:
-                rows = []
-                click.secho('\nAvailable exec targets:', fg='green')
-                click.secho('----------------------\n', fg='green')
-                number = 1
-                choices = []
-                for target in obj.ssh_targets:
-                    for container_name in obj.container_names:
-                        rows.append([
-                            number,
-                            click.style(target.tags['Name'], fg='cyan'),
-                            click.style(container_name, fg='yellow'),
-                            target.pk,
-                            target.ip_address
-                        ])
-                        choices.append((target, container_name))
-                        number += 1
-                click.secho(tabulate(rows, headers=['#', 'Instance', 'Container', 'Instance Id', 'IP']))
-                choice = click.prompt('\nEnter the number of the instance you want: ', type=int, default=1)
-                target, container_name = choices[choice - 1]
-        else:
-            target = obj.ssh_target
-            container_name = obj.container_name
-        if target is None:
-            raise self.RenderException(
-                '{}(pk="{}") has no instances available'.format(self.model.__class__, obj.pk)
-            )
+            running_tasks = sorted(obj.running_tasks, key=lambda x: x.ssh_target.tags['Name'])
+            rows = []
+            click.secho('\nAvailable exec targets:', fg='green')
+            click.secho('----------------------\n', fg='green')
+            number = 1
+            choices = []
+            for task in running_tasks:
+                for container in task.containers:
+                    rows.append([
+                        number,
+                        click.style(task.ssh_target.tags['Name'], fg='cyan'),
+                        click.style(container.name, fg='yellow'),
+                        click.style(container.version, fg='yellow'),
+                        task.ssh_target.pk,
+                        task.ssh_target.ip_address
+                    ])
+                choices.append((task.ssh_target, container.name))
+                number += 1
+            click.secho(tabulate(rows, headers=['#', 'Instance', 'Container', 'Version', 'Instance Id', 'IP']))
+            choice = click.prompt('\nEnter the number of the container you want: ', type=int, default=1)
+            target, container_name = choices[choice - 1]
         return target, container_name
 
 
@@ -117,7 +116,6 @@ to the container machine of a FARGATE {object_name}.
         )(function)
         function = click.option(
             '--choose/--no-choose',
-            '-v',
             default=False,
             help="Choose from all available targets for ssh, instead of having one chosen automatically."
         )(function)
@@ -172,14 +170,13 @@ Exec into a container in a {object_name} in AWS.
         )(function)
         function = click.option(
             '--choose/--no-choose',
-            '-v',
             default=False,
             help='Choose from all available targets for "docker exec", instead of having one chosen automatically.'
         )(function)
         function = click.argument('identifier')(function)
         function = command_group.command(
             'exec',
-            short_help='Exec into a container in AWS'.format(cls.model.__name__)
+            short_help='Exec into a container in AWS'
         )(function)
         return function
 
@@ -241,7 +238,6 @@ You can do this in two ways:
         )(function)
         function = click.option(
             '--choose/--no-choose',
-            '-v',
             default=False,
             help="Choose from all available targets for ssh, instead of having one chosen automatically."
         )(function)
@@ -308,39 +304,7 @@ You can do this in two ways:
             tunnel = tunnels[list(tunnels)[choice - 1]]
         return tunnel
 
-    @handle_model_exceptions
-    def tunnel(self, identifier, choose, local_port, host, host_port, verbose):
-        """
-        Establish an SSH tunnel from our machine through a Service instance to a host:port in AWS.
-
-        This is designed to be a multi-homed command so that we can put it at the top level:
-
-            deploy tunnel TUNNEL_NAME
-
-        or under the service group:
-
-            deploy service tunnel SERVICE_NAME TUNNEL_NAME
-            deploy service tunnel SERVICE_NAME --local-port=8888 --host=10.2.0.1 --host-port=3306
-
-
-        Three cases here:
-
-            * We're a command under the `service` command group
-              * In this case, identifier is a list with either one or entries
-                * If only one entry, that entry is the service name, and we expect the local_port, host and host_port
-                  arguments to be defined.
-                * If two entries, the first is the service name, and the second is the tunnel name.  We expect
-                  local_port, host and host_port to be None
-            * We're under the top level `cli` command group
-                * In this case, the identifier is a tunnel name, and local_port, host, and host_port are always None.
-
-        :param identifier Union[list(str), str]: either a SERVICE_NAME, TUNNEL_NAME pair, or either SERVICE_NAME
-                                                  or TUNNEL_NAME
-        :param choose bool: if True, present a list of instances available for tunneling through
-        :param local_port int: (optional) the local port to bind our end of the tunnel to
-        :param host int: (optional) the host in AWS on the other end of the tunnel
-        :param host_port int: (optional) the port on `host` on the other end of the tunnel
-        """
+    def _get_tunnel_config(self, identifier, local_port, host, host_port):
         if isinstance(identifier, tuple):
             # We're a command under the sub command group like service or cluster
             if identifier:
@@ -382,6 +346,42 @@ You can do this in two ways:
             else:
                 tunnel = self.get_tunnel()
             obj = tunnel
+        return tunnel, obj
+
+    @handle_model_exceptions
+    def tunnel(self, identifier, choose, local_port, host, host_port, verbose):
+        """
+        Establish an SSH tunnel from our machine through a Service instance to a host:port in AWS.
+
+        This is designed to be a multi-homed command so that we can put it at the top level:
+
+            deploy tunnel TUNNEL_NAME
+
+        or under the service group:
+
+            deploy service tunnel SERVICE_NAME TUNNEL_NAME
+            deploy service tunnel SERVICE_NAME --local-port=8888 --host=10.2.0.1 --host-port=3306
+
+
+        Three cases here:
+
+            * We're a command under the `service` command group
+              * In this case, identifier is a list with either one or entries
+                * If only one entry, that entry is the service name, and we expect the local_port, host and host_port
+                  arguments to be defined.
+                * If two entries, the first is the service name, and the second is the tunnel name.  We expect
+                  local_port, host and host_port to be None
+            * We're under the top level `cli` command group
+                * In this case, the identifier is a tunnel name, and local_port, host, and host_port are always None.
+
+        :param identifier Union[list(str), str]: either a SERVICE_NAME, TUNNEL_NAME pair, or either SERVICE_NAME
+                                                  or TUNNEL_NAME
+        :param choose bool: if True, present a list of instances available for tunneling through
+        :param local_port int: (optional) the local port to bind our end of the tunnel to
+        :param host int: (optional) the host in AWS on the other end of the tunnel
+        :param host_port int: (optional) the port on `host` on the other end of the tunnel
+        """
+        tunnel, obj = self._get_tunnel_config(identifier, local_port, host, host_port)
         if choose:
             target = self.get_ssh_target(tunnel)
         else:
@@ -403,3 +403,101 @@ You can do this in two ways:
             target.ip_address,
         ), fg='cyan')
         target.tunnel(tunnel, verbose=verbose)
+
+
+class ClickRunCommandCommandMixin(object):
+
+    @classmethod
+    def add_run_command_click_command(cls, command_group):
+        """
+        Build a fully specified click command for running a command on one or all instances in an ECS cluster and add it
+        to command group `command_group`.  Return the properly wrapped function object.
+
+        :param command_group function: the click command group function to use to register our click command
+
+        :rtype: function
+        """
+        def run_command(ctx, *args, **kwargs):
+            if cls.model.config_section is not None:
+                try:
+                    ctx.obj['config'] = get_config(**ctx.obj)
+                except ConfigProcessingFailed as e:
+                    ctx.obj['config'] = e
+            ctx.obj['adapter'] = cls()
+            click.secho(ctx.obj['adapter'].run_command(
+                kwargs['identifier'],
+                kwargs['command'],
+                kwargs['choose'],
+                kwargs['all'],
+                kwargs['verbose']
+            ))
+
+        pk_description = cls.get_pk_description()
+        run_command.__doc__ = """
+Run a shell command on an instance in a {object_name}.  If --all is passed, run the command
+on all instances in the {object_name}.
+
+
+{pk_description}
+
+""".format(pk_description=pk_description, object_name=cls.model.__name__)
+
+        function = print_render_exception(run_command)
+        function = click.pass_context(function)
+        function = click.option(
+            '--verbose/--no-verbose',
+            '-v',
+            default=False,
+            help="Show all SSH output."
+        )(function)
+        function = click.option(
+            '--choose/--no-choose',
+            default=False,
+            help="Choose from all available targets for ssh, instead of having one chosen automatically."
+        )(function)
+        function = click.option(
+            '--all/--no-all',
+            default=False,
+            help="Run the shell command on all instances in the {}".format(cls.model.__name__)
+        )(function)
+        function = click.argument('command', nargs=-1)(function)
+        function = click.argument('identifier', nargs=1)(function)
+        function = command_group.command(
+            'run',
+            short_help='Run a shell command on one or all instances of a {object_name}.'.format(
+                object_name=cls.model.__name__
+            )
+        )(function)
+        return function
+
+    @handle_model_exceptions
+    def run_command(self, identifier, command, choose, all_instances, verbose):
+        colors = [
+            'green',
+            'yellow',
+            'cyan',
+            'magenta',
+            'white',
+            'bright_green',
+            'bright_yellow',
+            'bright_cyan',
+            'bright_magenta',
+            'bright_white'
+        ]
+        colors_cycle = cycle(colors)
+        obj = self.get_object_from_aws(identifier)
+        command = ' '.join(command)
+        if not all_instances:
+            targets = [self.get_ssh_target(obj, choose=choose)]
+        else:
+            targets = obj.ssh_targets
+        for target in targets:
+            color = next(colors_cycle)
+            success, output = target.ssh_noninteractive(command, verbose=verbose, ssh_target=target)
+            if success:
+                for line in output.split('\n'):
+                    print('{}: {}'.format(click.style(target.name, fg=color), line))
+            else:
+                for line in output.split('\n'):
+                    line = click.style('ERROR: {}'.format(line), fg='red')
+                    print('{}: {}'.format(click.style(target.name, fg=color), line))
