@@ -9,3 +9,5 @@ cluster_exists = ClickClusterAdapter.add_exists_click_command(cluster_group)
 cluster_ssh = ClickClusterAdapter.add_ssh_click_command(cluster_group)
 cluster_tunnel = ClickClusterAdapter.add_tunnel_click_command(cluster_group)
 cluster_scale = ClickClusterAdapter.add_scale_instances_click_command(cluster_group)
+cluster_run = ClickClusterAdapter.add_run_command_click_command(cluster_group)
+cluster_running_tasks = ClickClusterAdapter.add_list_running_tasks_click_command(cluster_group)
