@@ -1178,10 +1178,13 @@ class TaskDefinition(TagsMixin, TaskDefinitionFARGATEMixin, SecretsMixin, Model)
 
     @property
     def version(self):
-        try:
-            return self.containers[0].data['image'].rsplit(':', 1)[1]
-        except IndexError:
-            return 'latest'
+        """
+        Return the version for the task definition.  We're cheating here by just returning the version of the first
+        container image, assuming that the first container will be the primary container for the TaskDefinition.
+
+        :rtype: str
+        """
+        return self.containers[0].version
 
     @property
     def secrets_prefix(self):
@@ -1329,6 +1332,13 @@ class ContainerDefinition(SecretsMixin, LazyAttributeMixin):
     def name(self):
         return self.data.get('name', None)
 
+    @property
+    def version(self):
+        try:
+            return self.data['image'].rsplit(':', 1)[1]
+        except IndexError:
+            return 'latest'
+
     @property
     def secrets_prefix(self):
         if self.secrets:
@@ -1601,6 +1611,10 @@ class InvokedTask(DockerMixin, Model):
             self.cache['task_definition'] = TaskDefinition.objects.get(self.data['taskDefinitionArn'])
         return self.cache['task_definition']
 
+    @property
+    def containers(self):
+        return self.task_definition.containers
+
     @property
     def instance(self):
         return self.container_instance.ec2_instance
@@ -1939,8 +1953,13 @@ class Service(TagsMixin, DockerMixin, SecretsMixin, Model):
         return self.cache['autoscaling_group']
 
     @property
-    def container_names(self):
-        return [c.name for c in self.task_definition.containers]
+    def containers(self):
+        """
+        This returns a list of  ``ContainerDefinition`` objects in the ``TaskDefinition`` for the PRIMARY deployment for
+        the service.  If you want the list of actual running containers for the service, use
+        ``self.running_containers``.
+        """
+        return self.task_definition.containers
 
     @property
     def ssh_tunnels(self):
