@@ -15,14 +15,14 @@ class AbstractSSHProvider(object):
         assert instance.__class__.__name__ == 'Instance', \
             '{}.instance must be an Instance object'.format(self.__class__.__name__)
         self.instance = instance
-        self.verbose = verbose
-        self.ssh_verbose_flag = '-vv' if self.verbose else ''
+        # If the caller specified --verbose, we send SSH the `-vv` flag.
+        self.ssh_verbose_flag = '-vv' if verbose else ''
 
-    def ssh(self, quiet=False):
+    def ssh(self, command=None):
         raise NotImplementedError
 
     def ssh_command(self, command):
-        return self.ssh(quiet=True, command=command)  # noqa
+        return self.ssh(command)
 
     def docker_exec(self):
         # FIXME: the "head -1" here crudely handles the case where we have multiple instances of the same container
@@ -39,11 +39,9 @@ class AbstractSSHProvider(object):
 
 class SSMSSHProvider(AbstractSSHProvider):
 
-    def ssh(self, quiet=False, command=None):
-        if quiet:
-            flags = "-q"
-        else:
-            flags = self.ssh_verbose_flag
+    def ssh(self, command=None):
+        # If the caller specified --verbose, have SSH print everything. Otherwise, have SSH print nothing.
+        flags = self.ssh_verbose_flag if self.ssh_verbose_flag else '-q'
         if not command:
             command = ''
         return 'ssh -t {} ec2-user@{} {}'.format(flags, self.instance.pk, shellescape.quote(command))
@@ -71,11 +69,9 @@ class BastionSSHProvider(AbstractSSHProvider):
         assert self.instance.bastion is not None, \
             '{}.instance has no bastion host'.format(self.__class__.__name__)
 
-    def ssh(self, quiet=False, command=None):
-        if quiet:
-            flags = "-q"
-        else:
-            flags = self.ssh_verbose_flag
+    def ssh(self, command=None):
+        # If the caller specified --verbose, have SSH print everything. Otherwise, have SSH print nothing.
+        flags = self.ssh_verbose_flag if self.ssh_verbose_flag else '-q'
         if not command:
             command = ''
         hop2 = "ssh {flags} -o StrictHostKeyChecking=no -A -t {instance} {command}".format(
@@ -83,7 +79,7 @@ class BastionSSHProvider(AbstractSSHProvider):
             instance=self.instance.ip_address,
             command=shellescape.quote(command)
         )
-        cmd = "ssh {flags} -o StrictHostKeyChecking=no -A -t ec2-user@{bastion} {hop2}".format(   # noqa:E501
+        cmd = "ssh {flags} -o StrictHostKeyChecking=no -A -t ec2-user@{bastion} {hop2}".format(
             flags=flags,
             hop2=shellescape.quote(hop2),
             bastion=self.instance.bastion.hostname,
@@ -92,7 +88,8 @@ class BastionSSHProvider(AbstractSSHProvider):
 
     def tunnel(self, local_port, target_host, host_port):
         interim_port = random.randrange(10000, 64000, 1)
-        cmd = 'ssh {flags} -L {local_port}:localhost:{interim_port} ec2-user@{bastion} ssh -L {interim_port}:{target_host}:{host_port} {instance}'.format(  # noqa:E501
+        cmd = ('ssh {flags} -L {local_port}:localhost:{interim_port} ec2-user@{bastion}'
+               ' ssh -L {interim_port}:{target_host}:{host_port} {instance}').format(
             flags=self.ssh_verbose_flag,
             local_port=local_port,
             interim_port=interim_port,
@@ -253,23 +250,31 @@ class DockerMixin(SSHMixin):
         pass
 
     @property
-    def container_names(self):
-        raise NotImplementedError
+    def running_tasks(self):
+        """
+        This should return a list of InvokedTask objects.
 
-    @property
-    def container_name(self):
-        return self.container_names[0]
+        :rtype: list(InvokedTask)
+        """
+        raise NotImplementedError
 
     def __init__(self, *args, **kwargs):
         self.provider_type = kwargs.pop('provider_type', 'bastion')
         super(DockerMixin, self).__init__(*args, **kwargs)
 
     def docker_exec(self, ssh_target=None, container_name=None, verbose=False):
+        if self.running_tasks:
+            if ssh_target is None:
+                ssh_target = self.running_tasks[0].ssh_target
+            if not container_name:
+                # Arbitrarily exec into the first container in our object
+                container_name = self.running_tasks[0].containers[0].name
         if ssh_target is None:
-            ssh_target = self.ssh_target
-        if not container_name:
-            container_name = self.container_names[0]
+            raise self.OperationFailed(f'{self.__class__.__name__}(pk={self.pk}) has no containers available.')
         provider = self.providers[self.ssh_proxy_type](ssh_target, verbose=verbose)
-        cmd = provider.docker_exec().format(self.task_definition.data['family'], container_name)  # noqa
+        cmd = provider.docker_exec().format(
+            self.task_definition.data['family'],
+            container_name.replace('_', '')
+        )
         cmd = provider.ssh_command(cmd)
         subprocess.call(cmd, shell=True)
